from typing import Tuple


def parse_bulk_upsert_results(
    results: list, data: list, salesforce_object: str, upsert_key: str
) -> Tuple[list, list]:
    """
    Parses the results of a bulk upsert call, collecting errors and successes

    # TODO: allow a custom serializer for errors
    # TODO: do something more with successes
    """
    assert len(results) == len(
        data
    ), f"Results ({len(results)}) and upload data ({len(data)}) have different lengths!"

    successes = list()
    errors = list()
    for i, result in enumerate(results):
        if result.get("success"):
            successes.append(result)
        for error in result.get("errors"):
            errors.append(
                {
                    "salesforce_object": salesforce_object,
                    "code": error.get("statusCode"),
                    "message": error.get("message"),
                    "upsert_key": upsert_key,
                    "upsert_key_value": data[i][upsert_key],
                    "object_json": data[i],
                }
            )
    return successes, errors


def extract_errors_from_results(results: list) -> list:
    """
    More general version of parse_bulk_upsert_results
    """
    errors = list()
    for result in results:
        success = result.get("success")
        if not success:
            errors += result.get("errors")
    return errors


import datetime

from django.conf import settings
from pathlib import Path


def get_temp() -> Path:
    assert hasattr(
        settings, "TEMP"
    ), "TEMP must be defined in your django settings; it should be a path to some folder on your local machine"
    return Path(settings.TEMP)


def get_iso() -> str:
    """
    Get an iso timestamp with less precision than .iso() returns, and no special characters
    """
    return datetime.datetime.now().strftime("%Y%m%dT%H%M")



import csv
from django_s3_csv_2_sfdc.utils import get_temp

from pathlib import Path
from typing import List, Tuple


def create_error_report(
    error_groups: List[list],
    report_path: Path = None,
    headers: List[str] = None,
) -> Tuple[Path, int]:
    """
    Takes in the errors from the output of parse_bulk_upsert_results and writes a report

    TEMP must be defined in your django settings
    """
    csv_rows = []
    if not headers:
        headers = [
            "salesforce_object",
            "code",
            "message",
            "upsert_key",
            "upsert_key_value",
            "object_json",
        ]
    csv_rows.append(headers)

    errors_count = 0
    for group in error_groups:
        for error in group:
            errors_count += 1
            csv_rows.append([error[header] for header in headers])

    if not report_path:
        report_path = get_temp() / "error_report.csv"

    with open(report_path, mode="w", newline="") as file:
        writer = csv.writer(
            file, delimiter=",", quotechar='"', quoting=csv.QUOTE_MINIMAL
        )
        for row in csv_rows:
            writer.writerow(row)

    return report_path, errors_count


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_s3_csv_2_sfdc']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.1.6,<4.0.0',
 'boto3>=1.17.3,<2.0.0',
 'simple-salesforce>=1.10.1,<2.0.0']

setup_kwargs = {
    'name': 'django-s3-csv-2-sfdc',
    'version': '0.2.10',
    'description': 'A set of helper functions for CSV to Salesforce procedures, with reporting in AWS S3, based in a Django project',
    'long_description': "# Overview\n\nA set of helper functions for CSV to Salesforce procedures, with reporting in AWS S3, based in a Django project.\nThe use case is extremely specific, but the helpers should be modular so they can be cherry-picked.\n\nTypical use case:\n\n- Receive an S3 event\n- Download the S3 object\n- Serialize the file into JSON\n- Bulk upsert the JSON data to Salesforce\n- Parse the results of the upsert for errors\n- Construct a CSV error report\n- Move the triggering S3 object to an archive folder\n- Push the error report to an error folder in the same bucket\n- Push an object to Salesforce that details information about the above execution\n\n# Example\n\n```python\nfrom django_s3_csv_2_sfdc.csv_helpers import create_error_report\nfrom django_s3_csv_2_sfdc.s3_helpers import download_file, respond_to_s3_event, upload_file\nfrom django_s3_csv_2_sfdc.sfdc_helpers import extract_errors_from_results\n\n\n# handler for listening to s3 events\ndef handler(event, context):\n    respond_to_s3_event(event, download_and_process)\n\n\ndef download_and_process(s3_object_key, bucket_name):\n    download_path = download_file(s3_object_key, bucket_name)\n\n    # This function contains your own biz logic; does not come from this library\n    results = serialize_and_push_to_sfdc(download_path)\n\n    sucesses, errors = parse_bulk_upsert_results(results)\n\n    report_path, errors_count = create_error_report([errors])\n\n    upload_file(report_path, bucket_name)\n```\n\nJust take what'cha need!\n",
    'author': 'Alex Drozd',
    'author_email': 'drozdster@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/brno32/django-s3-csv-2-sfdc',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import boto3
import os

from pathlib import Path

from django_s3_csv_2_sfdc.utils import get_temp, get_iso


def upload_file(
    local_path: Path, bucket: str, s3_key: Path = None, public_read: bool = False
):
    """Upload a file to an S3 bucket

    :param local_path: File to upload
    :param bucket: S3 Bucket to upload to
    :param s3_key: S3 object name. If not specified then local_path is used
    :param public_read: permissions
    """

    # If S3 s3_key was not specified, use local_path
    if s3_key is None:
        s3_key = local_path

    # S3 uses posix-like paths
    if type(s3_key) == Path:
        s3_key = s3_key.as_posix()
    # cast to string to get local filesystem's path
    local_path = str(local_path)

    s3_client = boto3.client("s3")
    if public_read:
        s3_client.upload_file(
            local_path, bucket, s3_key, ExtraArgs={"ACL": "public-read"}
        )
    else:
        s3_client.upload_file(local_path, bucket, s3_key)


def move_file(old_key: str, new_key: str, bucket: str):
    """
    Move a file within an S3 bucket by copying to a different path and deleting the original
    """
    s3_client = boto3.client("s3")
    copy_source = {"Bucket": bucket, "Key": old_key}
    s3_client.copy(copy_source, bucket, new_key)
    s3_client.delete_object(Bucket=bucket, Key=old_key)


def download_file(s3_object_key: str, bucket_name: str) -> Path:
    """
    Downloads a file from s3, dropping it in the temp directory
    following the pathing convention from the s3_object_key

        e.g., s3_object_key = archive/a_file.txt
        will drop it in

        %TEMP%/archive/a_file.txt

    TEMP must be defined in your django settings
    """
    tmp = get_temp()

    s3_client = boto3.client("s3")
    download_folder = tmp / os.path.dirname(s3_object_key)
    download_path = tmp / s3_object_key
    # spawn the nested folders without the os complaining
    Path(download_folder).mkdir(parents=True, exist_ok=True)
    s3_client.download_file(bucket_name, s3_object_key, str(download_path))

    return download_path


def timestamp_s3_key(
    s3_key: str, keep_folder: bool = False, timestamp: str = None
) -> str:
    dir_name = os.path.dirname(s3_key)
    file_name = os.path.basename(s3_key)
    name_and_extension = os.path.splitext(file_name)
    assert len(name_and_extension) == 2
    name = name_and_extension[0]
    extension = name_and_extension[1]

    if not timestamp:
        iso = get_iso()
    else:
        iso = timestamp

    timestamped_s3_key = f"{name}-{iso}{extension}"
    if keep_folder:
        return os.path.join(dir_name, timestamped_s3_key)
    return timestamped_s3_key


def respond_to_s3_event(event, callback, *args, **kwargs):
    """
    Use like this:
        def process_s3_event(s3_object_key, bucket_name):
            print(s3_object_key, bucket_name)

        def handler(event, context):
            respond_to_s3_event(event, process_s3_event)
    """
    records = event["Records"]
    for record in records:
        s3_data = record["s3"]
        bucket = s3_data["bucket"]
        bucket_name = bucket["name"]
        s3_object = s3_data["object"]
        s3_object_key = s3_object["key"]
        callback(s3_object_key, bucket_name, *args, **kwargs)